# surface_density_plot.py
#
# This script creates Figure 2 in in Rowlinson et al. 2022
# The script loads each image in the dataset, divides the image area into
# annuli, record the sigma clipped rms in each annulus and the area enclosed by
# the annulus. The rms values are then collated into bins with the total area 
# recorded for each bin. Then, this is converted into a transient surface
# density and the sensitivity of the each bin is calculated using the
# detection threshold. Finally, the result is plotted alongside other
# surveys conducted at this observing frequency

from astropy.io import fits
from astropy.stats import sigma_clipped_stats
from astropy.coordinates import SkyCoord
from astropy import units as u
from regions import CirclePixelRegion, PixCoord, CircleAnnulusPixelRegion
import numpy as np
import matplotlib
matplotlib.use('Agg')
import matplotlib.pyplot as plt
import matplotlib.mlab as mlab
from scipy.stats import norm
import glob
import os
from photutils import aperture
import pandas as pd
import toolsHIGH


sig_clip=2. # median sigma clipping value to remove sources
srch_radius=2495. #Search radius in pixels - same as in TraP
img_cntr=[5120,5120] # image centre in pixels
pixelsize=0.000305555555555556 # pixel size in degrees from the fits header
nbins = 100 # number of annuli bins to use
sourceExtrSig=5.56 # the source detection threshold used in TraP to search for transients
time_step = 7 # the number of days typically between snapshots

increment = int(srch_radius/nbins) # The width of each annulus in pixels
area1pix = pixelsize**2. # area of 1 pixel in degrees

path_to_imgs = "/path/to/images/*.fits" # the path to the images on your computer system

##### n.b. this method is too slow for large datasets. Recommend using a random sample of e.g. 10%
##### and then multiply the cumulative area by an equivalent factor (10% means multiply by 10)
##### this is not precise but does give the approximate correct answer assuming the 10% is
##### representative of the full sample of images



if os.path.exists("rmsAreaData.csv"): # The script outputs this file on first run. The data file for Figure 2 is included with this script
    print 'reading data file'
    rms_bins = pd.read_csv("rmsAreaData.csv", index_col=0)
    print 'data read'

else:
    images=glob.glob(os.path.expanduser(path_to_imgs)) # find all the images
    rmsData = []

    for img in images: # loop through all the images
        print img
        hdu = fits.open(img) 
        fitsdata = hdu[0].data[0][0] # obtain the pixel values in every image
        radius_tmp = 0

        for x in range(nbins): # loop through each annulus
            radius_inner = radius_tmp + x*increment 
            radius_outer = radius_tmp + (x+1)*increment
            skyregion = aperture.CircularAnnulus(img_cntr, radius_inner, radius_outer) # define a sky region for the annulus 
            areaindeg = skyregion.area() * area1pix # calculate the area covered by the annulus
            masks = skyregion.to_mask(method='center')
            annulus_data = masks[0].multiply(fitsdata)
            mask = masks[0].data # obtain all the pixel values within the annulus
            annulus_data_1d = annulus_data[mask > 0]
            annulus_data_1d = annulus_data_1d[annulus_data_1d != 0]
            _, median_sigclip, median_stdev = sigma_clipped_stats(annulus_data_1d) # sigma clip to remove sources
            rmsData.append([median_stdev, areaindeg]) # record the final rms and the area included

    rmsData = pd.DataFrame(data=rmsData, columns=['rms','area'])
    min_rms = min(rmsData.rms)
    max_rms = max(rmsData.rms)
    rms_range = np.linspace(min_rms,max_rms,num=100)

    rms_bins = [[rms_range[x],rms_range[x+1],0,0,0,0] for x in range(len(rms_range)) if x < len(rms_range)-1] # set up the rms bins for final plot


    # for each row in the array rmsData, the following determines which rms bin it falls into and then adds the area for each bin
    for r in range(len(rms_bins)):
        for index, row in rmsData.iterrows():
            if row['rms'] >= rms_bins[r][0] and row['rms'] < rms_bins[r][1]:
                rms_bins[r][2] = rms_bins[r][2] + row['area']

    # For the final plot we want the cumulative area as a function of the rms, so here we calculate the cumulative values
    cumulativearea=0
    for r in range(len(rms_bins)):
        cumulativearea = cumulativearea + rms_bins[r][2] # the cumulative area
        rms_bins[r][3]=cumulativearea
        rms_bins[r][4] = 3. / cumulativearea # the transient surface density assuming a non detection at 95% confidence using Poissonian statistics
        rms_bins[r][5] = rms_bins[r][1] * sourceExtrSig # the 

    rms_bins = pd.DataFrame(data=rms_bins, columns=['rms_min','rms_max','area','cumulative','surfDensity','flxDensity'])
    rms_bins.to_csv('rmsAreaData.csv')

print rms_bins

# Normal rates plot
ratesData = toolsHIGH.read_datafile('ratesDataHIGH.csv') # a datafile containing the parameters for the other transient searches published at this frequency
TMPdata=[[float(x[0]), float(x[1]), float(x[2]), x[3], x[4]] for x in ratesData]
toolsHIGH.plotfig_rates2(TMPdata,0,1, 'Limiting Flux Density (Jy)', r'Surface density (deg$^{-2}$)', 'rates_all',False, rms_bins,time_step) # create the final plot


exit()

